LL_wlogit <- function(Y,X,coefficients,w,long=FALSE){
  #DESCRIPTION
  # Calculates the log likelihood for a weighted logit at given coefficients (from Greene)
  #-------------------------------------------------------------------------------
  #USAGE
  # LL_wlogit(Y,X,coefficients,w,long=FALSE)
  #-------------------------------------------------------------------------------
  #ARGUMENTS
  # Y:            matrix of the LHS variable
  # X:            matrix of RHS variables
  # coefficients: vector of coefficients.
  # w:            vector of observation weights
  # long:         boolean indicating if additional output should be returned
  #-------------------------------------------------------------------------------
  #OUTPUT
  # either negative log likelihood (if long = FALSE)
  # or list with positive log likelihood and a set of additional output

  q <- 2*Y-1
  Xbeta <- X%*%coefficients
  qXbeta <- q*Xbeta
  qGamma <- exp(qXbeta)/(1+exp(qXbeta))
  LL <- sum(w*log(qGamma))
  
  if (!long){return(-LL)}
  else{
    out <- NULL
    out$LL <- LL
    out$coefficients <- coefficients
    out$B <- sum((Y==0)*(Xbeta>0))
    out$C <- sum((Y==1)*(Xbeta<0))
    out$fitted.values <- c(exp(Xbeta)/(1+exp(Xbeta)))
    return(out)
  }
}