LL_wprobit <- function(Y,X,coefficients,w,long=FALSE){
  #DESCRIPTION
  # Calculates the log likelihood for a weighted probit at given coefficients (from Greene, p. 793)
  #-------------------------------------------------------------------------------
  #USAGE
  # LL_wprobit(Y,X,coefficients,w,long=FALSE)
  #-------------------------------------------------------------------------------
  #ARGUMENTS
  # Y:            matrix of the LHS variable
  # X:            matrix of RHS variables
  # coefficients: vector of coefficients.
  # w:            vector of observation weights
  # long:         boolean indicating if additional output should be returned
  #-------------------------------------------------------------------------------
  #OUTPUT
  # either negative log likelihood (if long = FALSE)
  # or list with positive log likelihood and a set of additional output
  
  
#   print(dim(X))
#   print(length(coefficients))
#   print(coefficients)
  q <- 2*Y-1
  Xbeta <- X%*%coefficients
  qXbeta <- q*Xbeta
  LL <- sum(w*log(pnorm(qXbeta)))
  
  if (!long){return(-LL)}
  else{
    out <- NULL
    out$LL <- LL
    out$coefficients <- coefficients
    out$B <- sum((Y==0)*(Xbeta>0))
    out$C <- sum((Y==1)*(Xbeta<0))
    out$fitted.values <- c(pnorm(Xbeta))
    return(out)
  }
}