construct_restable.fc <- function(res_array,muvec,p_mat,b_U_Sarlin=TRUE,savenme){
  # Aggregate result tables for recursive exercises, where time is another dimension
  #
  # Gregor von Schweinitz and Peter Sarlin, 2018-07-25
  
  methods <- dimnames(res_array)$method
  T <- dimnames(res_array)$T
  resnames <- c("A","B","C","D","L","Ua","Ur","p")
  restable.arr <- array(dim=c(length(muvec),length(methods),length(resnames)),
                    dimnames=list(mu=muvec,method=methods,res=resnames))
  
  combnames <- expand.grid(methods,muvec)
  
  restable <- array(dim=c(length(muvec)*length(methods),length(resnames)+2),
                    dimnames=list(paste(combnames[,1],combnames[,2],sep="."),res=c("mu","method",resnames)))
  restable <- as.data.frame(restable)
  for (mu in muvec){
    for (m in methods){
      A <- sum(res_array[,m,as.character(mu),"A_out"],na.rm=TRUE)
      B <- sum(res_array[,m,as.character(mu),"B_out"],na.rm=TRUE)
      C <- sum(res_array[,m,as.character(mu),"C_out"],na.rm=TRUE)
      D <- sum(res_array[,m,as.character(mu),"D_out"],na.rm=TRUE)
      N <- A+B+C+D
      P1 <- (A+C)/N # Being in EWW: (A+C)/N
      P2 <- (B+D)/N # not beeing in EWW
      P1T1 <- C/N # out of for calculation cause of speed
      P2T2 <- B/N
      T1 <- C/(A+C)
      T2 <- B/(B+D)
      
      if (b_U_Sarlin){
        L <- mu*P1T1+(1-mu)*P2T2  # Loss function
        Ua <- min(mu*P1,(1-mu)*P2)-L # absolute utility
        Ur <- Ua/(min(mu*P1,(1-mu)*P2)) # rel utility
      }else{
        L <- mu*T1 + (1-mu)*T2
        Ua <- min(mu,1-mu)-L
        Ur <- Ua/min(mu,1-mu)
      }
      prob <- p_mat[as.character(mu),m]
      restable.arr[as.character(mu),m,] <- c(A,B,C,D,L,Ua,Ur,prob)
      
      pos <- which(combnames[,1]==m & combnames[,2]==mu)
      restable[pos,] <- c(mu,m,A,B,C,D,L,Ua,Ur,prob)
    }
  }
  
  write.csv(restable,file=savenme)
  return(restable)
}