findThreshold <- function(y,p_y, muvec, lambda_set,b_U_Sarlin = TRUE){
  # DESCRIPTION
  # Calculates the optimal threshold lambda for different preferences
  # Takes into account restrictions on allowed thresholds
  #--------------------------------
  # INPUT
  # y           binary event vector
  # p_y         continuous probability vector on which signaling threshold is searched
  # muvec       set of preferences for which thresholds are to be used
  # lambda_set  thresholds to be accepted (given as percentiles of the data)
  # b_U_sarlin  binary switch between usefulness of Sarlin and Alessi-Detken
  #-------------------------------
  # OUTPUT
  # Result matrix, one row per mu
  # with elements threshold, confusion matrix, loss function, absolute/relative usefulness
  #
  # Gregor von Schweinitz and Peter Sarlin, 2018-07-25
  
  
  resvec <- c("lambda","A","B","C","D","L","Ua","Ur")
  res_threshold <- mat.or.vec(length(muvec),length(resvec))
  rownames(res_threshold) <- muvec
  colnames(res_threshold) <- resvec
  
  N <- length(y[!is.na(y)]) # length of observations in Crisis  
  P1 <- sum(y==1, na.rm=TRUE)/N # Being in EWW: (A+C)/N
  P2 <- sum(y==0, na.rm=TRUE)/N # not beeing in EWW
#   if(is.null(P1)){P1 <- sum(y==1, na.rm=TRUE)/N} # Being in EWW: (A+C)/N
#   if(is.null(P2)){P2 <- sum(y==0, na.rm=TRUE)/N} # not beeing in EWW
  
  N_lambda <- length(lambda_set) # Running Index for lambda
  A <- rep(0,N_lambda)    # create the vectors
  B <- rep(0,N_lambda)    
  C <- rep(0,N_lambda)
  D <- rep(0,N_lambda)
  L <- rep(0,N_lambda)
  Ua <- rep(0,N_lambda)
  Ur <- rep(0,N_lambda)
  
  for (i in 1:N_lambda){
    lambda <- lambda_set[i] # set up lambda from 0.01 to 0.99 by 0.01 each T1P1
    A[i] <- sum(p_y[y==1] > lambda, na.rm=TRUE)
    B[i] <- sum(p_y[y==0] > lambda, na.rm=TRUE)
    C[i] <- sum(p_y[y==1] <= lambda, na.rm=TRUE)
    D[i] <- sum(p_y[y==0] <= lambda, na.rm=TRUE)
  }    
  lambda <- lambda_set
  P1T1 <- C/N # out of for calculation cause of speed
  P2T2 <- B/N
  T1 <- C/(A+C)
  T2 <- B/(B+D)
  
  for (m in 1:length(muvec)){
    mu=muvec[m]
    if (b_U_Sarlin){
      L <- mu*P1T1+(1-mu)*P2T2  # Loss function
      Ua <- min(mu*P1,(1-mu)*P2)-L # absolute utility
      Ur <- Ua/(min(mu*P1,(1-mu)*P2)) # rel utility
    }else{
      L <- mu*T1 + (1-mu)*T2
      Ua <- min(mu,1-mu)-L
      Ur <- Ua/min(mu,1-mu)
    }
    optrow <- which(L==min(L))[1]
    for (k in resvec){res_threshold[m,k] <- get(k)[optrow]}
  }

  return(res_threshold)
}