hess_wlogit <- function(Y,X,coefficients,w,type="sandwich"){
  #DESCRIPTION
  # Calculates the hessian of the log likelihood for a weighted logit at given coefficients (from Greene)
  #-------------------------------------------------------------------------------
  #USAGE
  # hessiann_wlogit(Y,X,coefficients,w,type="sandwich")
  #-------------------------------------------------------------------------------
  #ARGUMENTS
  # Y:            matrix of the LHS variable
  # X:            matrix of RHS variables
  # coefficients: vector of coefficients.
  # w:            vector of observation weights
  # type:         string of values "H", "B", "sandwich", indicating which type of hessian should be calculated
  #               Greene recommends a sandwich estimator for the weighted case.
  #               The R-estimator of the hessian is most closely related to the H estimator
  #-------------------------------------------------------------------------------
  #OUTPUT
  # negative hessian (for optimization of negative likelihood)
  
  typematch <- grep(type,c("H","B","sandwich"))
  if (length(typematch)==0){
    error("Type has to be one of H, B, or sandwich")
  }
  
  q <- 2*Y-1
  Xbeta <- X%*%coefficients
  Gamma <- exp(Xbeta)/(1+exp(Xbeta))
#   qXbeta <- q*Xbeta
#   lambda <- q*dnorm(qXbeta) / pnorm(qXbeta)
  facH <- -w*Gamma*(1-Gamma)
  facB <- w*(Y-Gamma)^2
  
  N <- length(coefficients)
  T <- length(Y)
  H <- mat.or.vec(N,N)
  B <- mat.or.vec(N,N)
  
  for (i in 1:T){
    mat <- X[i,] %*% t(X[i,])
    #     print(dim(mat))
    H <- H+facH[i]*mat
    B <- B+facB[i]*mat
  }
  #   Hinv <- solve(H)
  
  #   hess_wprobit <- Hinv %*% B %*% Hinv
  
  if (typematch==1) {hess_wprobit <- H}
  if (typematch==2) {hess_wprobit <- B}
  if (typematch==3) {hess_wprobit <- H %*% solve(B) %*% H}
  
  return(-hess_wprobit)
}