plot_U_bs_realdata <- function(filename,main,res_mean,res_array,ylab="Ur",xlab="mu",plotconf=FALSE,type="pdf"){
  # Plots probability of outperformance in real data from bootstrap exercise
  #
  # Gregor von Schweinitz and Peter Sarlin, 2018-07-25
  
  if (type=="pdf") pdf(paste(filename,".pdf",sep=""),width=12,height=9)
  if (type=="tiff") tiff(filename = paste(filename,".tiff",sep=""),width=12,height=9,units="in",compression="none",res=300)
  
  usecols <- reportcols-1
  lwd <- c(1.5,1.5,2,rep(1,6))
  col <- rep(1,9)
  lty <- rep(c(2,3,1),3)
  legend <- paste(dimnames(res_array)$method[usecols],"(incl 90% conf)")
  lsort <- c(1:3)
  layout(matrix(c(1,2,3), ncol=1, byrow=TRUE), heights=c(4,4,1))
  par(mai=c(0.5,0.7,0.5,0.5))
  x <- as.numeric(dimnames(res_array)$mu)
  if (plotconf){
    y <- cbind(res_array[,usecols,ylab],
                            res_array[,usecols,paste(ylab,"_0.05",sep="")],
                            res_array[,usecols,paste(ylab,"_0.95",sep="")])
  }else{
    y <- res_mean[,reportcols]
  }
  matplot(x=x,y=y,type="l",main=main[1],xlab=xlab,ylab=ylab,lty=lty,col=col,lwd=lwd,ylim=c(-0.2,0.7),cex.main=1.7,cex.lab=1.7,cex.axis=1.5)
  abline(a=0,b=0)
  
  usecols2 <- usecols[2:3]
  lsort2 <- lsort[2:3]
  y <- res_array[,usecols2,"p"]
  matplot(x=x,y=y,type="l",main=main[2],xlab=xlab,ylab="probability",ylim=c(0,1),lty=lty[lsort2],col=col[lsort2],lwd=lwd[lsort2],cex.main=1.7,cex.lab=1.7,cex.axis=1.5)
  abline(h=0.5)
  
  par(mai=rep(0,4))
  plot.new()
  legend(x="center", ncol=3,legend=legend[lsort],lty=lty[lsort],col=col[lsort],lwd=lwd[lsort],cex=1.5)
  dev.off()
  
}