plot_crisisfreq <- function(freq_cr,savename=NULL,samplesplit=NULL,ylab="freq (crisis)",samplemeans = NULL){
  # DESCRIPTION
  # Creates a barplot of event frequencies over time, with an optional split into in-sample and out-of-sample
  # Adds sample averages of subsamples as lines
  # ------------------------
  # INPUT
  # freq_cr:      matrix of dates and crisis frequencies
  # savenme:      file name of output plot (without ".pdf")
  # samplesplit:  Optional date of sample split
  # ylab:         Optional label of y-axis
  # samplesplit:  Optional sample means, to be added as lines
  #
  # Gregor von Schweinitz and Peter Sarlin, 2018-07-25
  
  
  if(!is.null(savename)){pdf(paste(savename,".pdf",sep=""),width=11,height=6)}
  
  layout(matrix(c(1:2),ncol=1),heights=c(5,1))
  par(mai=c(0.5,0.8,1,0.3))
  x <- 1:dim(freq_cr)[1]
  y <- freq_cr[,2]
  m<-barplot(y,ylim=c(0,1),xpd=FALSE,ylab=ylab)
  if (!is.null(samplesplit)){
    vpos <- mean(m[which(freq_cr[,1]==samplesplit)-c(0:1)])
    if (is.null(samplemeans)){
      fpre <- mean(freq_cr[m<vpos,2])
      fpost <- mean(freq_cr[m>vpos,2])
    }else{
      fpre <- samplemeans[1]
      fpost <- samplemeans[2]
    }
    
    abline(v=vpos)
    text(x=c(vpos-0.04*max(m),vpos+0.04*max(m)),y=c(0.8,0.8),labels=c("is","oos"),pos=3,cex=2)
    # segments(x0=0,x1=vpos,y0=mean(freq_cr[m<vpos,2]),y1=mean(freq_cr[m<vpos,2]))
    segments(x0=c(0,vpos),x1=c(vpos,max(m)+min(m)),y0=c(fpre,fpost),y1=c(fpre,fpost),lwd=2)
    # segments(x0=vpos,x1=1,y0=mean(freq_cr[m>vpos,2]),y1=mean(freq_cr[m>vpos,2]))
  }
  axis(1,at=m,labels=format(as.Date(freq_cr[,1]),cex.axis=1.5))
  
  par(mai=rep(0,4))
  plot.new()
  legend("center",legend=c("sample mean"),lwd=2)
  if(!is.null(savename)){dev.off()}
}