plot_realtime <- function(res_array,filename1,column,muvec1,ylab,filename2,method,legendtext,var,muvec2,filename3,freq_cr,b_U_Sarlin = TRUE,type="pdf"){
  # Plots developments in recursive exercises of
  #   - Thresholds (heatmap)
  #   - Usefulness
  #   - Crisis frequencies (barplot) together with thresholds (line)
  #
  # Gregor von Schweinitz and Peter Sarlin, 2018-07-25
  
  
  library("plot3D")
  x <- 1:length(dimnames(res_array)[[1]])
  if (type=="pdf") pdf(paste(filename1,".pdf",sep=""),width=11,height=5)
  if (type=="tiff") tiff(filename=paste(filename1,".tiff",sep=""),width=11,height=5,units="in",compression="none",res=300)
  
  image2D(x=x,y=muvec1,z=res_array[,column,,"lambda"],zlim=c(0,1),xaxt="n",xlab="Time",ylab=ylab,cex.axis=1.2,cex.lab=1.5,cex.main=1.5)
  
  axis(1,at=x,labels=format(as.Date(dimnames(res_array)[[1]])),cex.axis=1.2)
  dev.off()



  if (type=="pdf") pdf(paste(filename2,".pdf",sep=""),width=11,height=4.3)
  if (type=="tiff") tiff(filename=paste(filename2,".tiff",sep=""),width=11,height=4.3,units="in",compression="none",res=300)
  
  mu <- muvec2
  ncol <- length(mu)
  layout(matrix(c(1:ncol,rep(ncol+1,ncol)), ncol=ncol, byrow=TRUE), heights=c(3.3,1))
  par(mai=c(0.5,0.8,1,0.3))
  main = paste(ylab,"=",mu,sep="")
  x <- 1:length(dimnames(res_array)[[1]])
  for (p in 1:ncol){
   posmu <- which(as.numeric(dimnames(res_array)[[3]])==mu[p])
   y <- res_array[c(1:9),method,posmu,var]
   matplot(1:length(x[1:9]),y,type="l",lwd=2,lty=c(2,3,1),col=c(1,1,1),xlab="Time",xaxt="n",ylab="abs. U",main=main[p],mu[p],cex.main=1.7,cex.axis=1.5,cex.lab=1.7,ylim=c(-0.2,0.1))
   abline(a=0,b=0)
   axis(1,at=x,labels=format(as.Date(dimnames(res_array)[[1]])),cex.axis=1.5)
  }
  
  par(mai=rep(0,4))
  plot.new()
  legend(x="center", ncol=3,legend=legendtext,lwd=2,lty=c(2,3,1),col=c(1,1,1),cex=1.5)
  dev.off()
  
  if (type=="pdf") pdf(paste(filename3,".pdf",sep=""),width=11,height=6)
  if (type=="tiff") tiff(filename=paste(filename3,".tiff",sep=""),width=11,height=6,units="in",compression="none",res=300)
  
  layout(matrix(c(1,2), ncol=1, byrow=TRUE), heights=c(5,1))
  par(mai=c(0.5,0.8,1,0.3))
  x <- 1:length(dimnames(res_array)[[1]])
  y <- res_array_fc[,column,as.character(muvec2),"lambda"]
  
  par(mar=c(6, 4, 4, 4))
  m<-barplot(freq_cr,ylim=c(0,1),xpd=FALSE,ylab="frequency of crises")
  axis(1,at=m,labels=format(as.Date(dimnames(res_array)[[1]])),cex.axis=1.5)
  xlims <- par('usr')[1:2]
  par(new=T)
  matplot(m,y,type="l",lwd=2,lty=c(1:length(muvec2)),col=1,axes=F,
          xlab="",ylab="",main="Pre-Crisis frequency and thresholds",cex.main=1.7,cex.axis=1.5,cex.lab=1.7)
  mtext("thresholds",side=4,line=1.5)
  
  par(mai=rep(0,4))
  plot.new()
  if (b_U_Sarlin){
    type<- "mu"
  }else{
    type<-"theta"
  }
  legend(x="center", ncol=length(muvec2),legend=paste(type,muvec2),lwd=2,lty=c(1:length(muvec2)),col=rep(1,length(muvec2)),cex=1.5)
  dev.off()
}