plot_simdata <- function(plotdata,savenme.part,plotsave=TRUE,onefile=TRUE){
  # Plots output from simulations of data generating processes
  
  data <- plotdata
  mu <- c(0.2,0.5,0.8,0.95)
  ncol <- length(mu)
  method.bm <- c("Probit opt threshold","Logit opt threshold")
  method.p <- c("Probit opt threshold","Weighted probit","Probit set threshold")
  method.l <- c("Logit opt threshold","Weighted logit","Logit set threshold")
  legend.bm <- c("Probit, opt threshold","Logit, opt threshold")
  legend.p <- c("Probit, opt threshold","Weighted probit","Probit, set threshold")
  legend.l <- c("Logit, opt threshold","Weighted logit","Logit, set threshold")
  
  if (plotsave & onefile) pdf(paste("plots_",savenme.part,".pdf",sep=""),width=13,height=9)
  
  if (plotsave & !onefile) pdf(paste("lambdaplot_",savenme.part,".pdf",sep=""),width=13,height=5)
  
  var <- "lambda_m"
  colvar <- which(colnames(data)==var)
  
  layout(matrix(c(1:ncol,rep(ncol+1,ncol)), ncol=ncol, byrow=TRUE), heights=c(4, 1))
  par(mai=c(0.5,0.6,1,0.25))
  for (p in 1:ncol){
    rows <- which((data[,"method"]==method.bm[1]) & (data[,"mu"]==mu[p]))
    x <- data[rows,"T"]
    y <- data[rows,colvar]
    if (length(method.bm)>1){
      for (i in 2:length(method.bm)){
        rows <- which((data[,"method"]==method.bm[i]) & (data[,"mu"]==mu[p]))
        print(length(rows))
        y <- cbind(y,data[rows,colvar])
      }
    }
    pos <- c(1,ceiling(length(x)/2),length(x))
    matplot(1:length(x),y,type="l",lwd=c(2,2),lty=c(1,2),col=c(1,1),xlab="observations",ylab="lambda",xaxt='n',main=paste("mu =",mu[p]),cex.main=1.7,cex.axis=1.5,cex.lab=1.7,ylim=c(0,1))
    abline(a=1-mu[p],b=0)
    axis(1,at=pos,labels=x[pos],cex.axis=1.2)
  }
  par(mai=rep(0,4))
  plot.new()
  legend(x="center", ncol=2,legend=legend.bm,lwd=2,lty=c(1,2),col=c(1,1),cex=1.5)
  if (plotsave & !onefile) dev.off()
  
  # -------------------------------------------------------------------------
  # Lambdaplot_conf
  data<- plotdata
  if (plotsave & !onefile) pdf(paste("lambdaplot_conf_",savenme.part,".pdf",sep=""),width=13,height=4.5)
  var <- c("lambda_m","lambda_05","lambda_95")
  colvar <- match(var,colnames(data))
  
  ncol <- length(mu)
  layout(matrix(c(1:ncol,rep(ncol+1,ncol)), ncol=ncol, byrow=TRUE), heights=c(4, 1))
  par(mai=c(0.5,0.6,1,0.25))
  for (p in 1:ncol){
    rows <- which((data[,"method"]==method.bm[1]) & (data[,"mu"]==mu[p]))
    x <- data[rows,"T"]
    y <- data[rows,colvar]
    if (length(method.bm)>1){
      for (i in 2:length(method.bm)){
        rows <- which((data[,"method"]==method.bm[i]) & (data[,"mu"]==mu[p]))
        y <- cbind(y,data[rows,colvar])
      }
    }
    pos <- c(1,ceiling(length(x)/2),length(x))
    matplot(1:length(x),y,type="l",lwd=c(2,1,1,2,1,1),lty=c(1,1,1,2,2,2),col=1,xlab="observations",ylab="lambda",xaxt='n',main=paste("mu =",mu[p]),cex.main=1.7,cex.axis=1.5,cex.lab=1.7,ylim=c(0,1))
    abline(a=1-mu[p],b=0)
    axis(1,at=pos,labels=x[pos],cex.axis=1.2)
  }
  par(mai=rep(0,4))
  plot.new()
  legend(x="center", ncol=2,legend=legend.bm,lwd=2,lty=c(1,2),col=c(1,1),cex=1.5)
  if (plotsave & !onefile) dev.off()
  
  # -------------------------------------------------------------------------
  # R2/R2adj
  data<- plotdata
  for (k in 1:2){
    if (k==1){
      if (plotsave & !onefile) pdf(paste("R2adj_probit_",savenme.part,".pdf",sep=""),width=13,height=5)
      method <- method.p
      legendtext <- legend.p
    }else{
      if (plotsave & !onefile) pdf(paste("R2adj_logit_",savenme.part,".pdf",sep=""),width=13,height=5)
      method <- method.l
      legendtext <- legend.l
    }
    
    var <- c("R2adj")
    colvar <- which(colnames(data)==var)
    
    layout(matrix(c(1:ncol,rep(ncol+1,ncol)), ncol=ncol, byrow=TRUE), heights=c(4, 1))
    par(mai=c(0.5,0.6,1,0.25))
    for (p in 1:ncol){
      rows <- which((data[,"method"]==method[1]) & (data[,"mu"]==mu[p]))
      x <- data[rows,"T"]
      y <- data[rows,colvar]
      if (length(method)>1){
        for (i in 2:length(method)){
          rows <- which((data[,"method"]==method[i]) & (data[,"mu"]==mu[p]))
          y <- cbind(y,data[rows,colvar])
        }
      }
      pos <- c(1,ceiling(length(x)/2),length(x))
      matplot(1:length(x),y,type="l",lwd=2,lty=c(2,3,1),col=1,xlab="observations",ylab="R2",xaxt='n',main=paste("mu =",mu[p]),cex.main=1.5,cex.axis=1.2,cex.lab=1.5)
      axis(1,at=pos,labels=x[pos],cex.axis=1.2)
    }
    par(mai=rep(0,4))
    plot.new()
    legend(x="center", ncol=3,legend=legendtext,lwd=2,lty=c(2,3,1),col=1,cex=1.5)
    if (plotsave & !onefile) dev.off()
  }
  
  # -------------------------------------------------------------------------
  # utility (relative)
  
  for (k in 1:2){
    if (k==1){
      if (plotsave & !onefile) pdf(paste("Ur_",savenme.part,"_probit.pdf",sep=""),width=13,height=9)
      method <- method.p
      legendtext <- legend.p
    }else{
      if (plotsave & !onefile) pdf(paste("Ur_",savenme.part,"_logit.pdf",sep=""),width=13,height=9)
      method <- method.l
      legendtext <- legend.l
    }
    var <- c("Ur_in_m","Ur_out_m")
    mainvec <- c("is", "oos")
    
    layout(matrix(c(1:(2*ncol),rep(2*ncol+1,ncol)), ncol=ncol, byrow=TRUE), heights=c(4,4,1))
    par(mai=c(0.5,0.6,1,0.25))
    for (case in 1:2){
      main = paste("mu=",mu," (",mainvec[case],")",sep="")
      colvar <- which(colnames(data)==var[case])
      for (p in 1:ncol){
        rows <- which((data[,"method"]==method[1]) & (data[,"mu"]==mu[p]))
        x <- data[rows,"T"]
        y <- data[rows,colvar]
        if (length(method)>1){
          for (i in 2:length(method)){
            rows <- which((data[,"method"]==method[i]) & (data[,"mu"]==mu[p]))
            y <- cbind(y,data[rows,colvar])
          }
        }
        pos <- c(1,ceiling(length(x)/2),length(x))
        #     matplot(1:length(x),y,type="l",lwd=2,lty=c(2,3,1),col=c(1,1,1),xlab="observations",ylab="rel. U",xaxt='n',main=main[p],mu[p],cex.main=1.5,cex.axis=1.2,cex.lab=1.5,ylim=c(-0.4,0.6))
        matplot(1:length(x),y,type="l",lwd=2,lty=c(2,3,1),col=c(1,1,1),xlab="observations",ylab="rel. U",xaxt='n',main=main[p],mu[p],cex.main=1.5,cex.axis=1.2,cex.lab=1.5)
        abline(a=0,b=0)
        axis(1,at=pos,labels=x[pos],cex.axis=1.2)
      }
    }
    par(mai=rep(0,4))
    plot.new()
    legend(x="center", ncol=3,legend=legendtext,lwd=2,lty=c(2,3,1),col=c(1,1,1),cex=1.5)
    if (plotsave & !onefile) dev.off()
  }
  
  # -------------------------------------------------------------------------
  # Usefulness (absolute)
  for (k in 1:2){
    if (k==1){
      if (plotsave & !onefile) pdf(paste("Ua_",savenme.part,"_probit.pdf",sep=""),width=13,height=9)
      method <- method.p
      legendtext <- legend.p
    }else{
      if (plotsave & !onefile) pdf(paste("Ua_",savenme.part,"_logit.pdf",sep=""),width=13,height=9)
      method <- method.l
      legendtext <- legend.l
    }
    
    var <- c("Ua_in_m","Ua_out_m")
    mainvec <- c("is", "oos")
    
    layout(matrix(c(1:(2*ncol),rep(2*ncol+1,ncol)), ncol=ncol, byrow=TRUE), heights=c(4,4,1))
    par(mai=c(0.5,0.6,1,0.25))
    for (case in 1:2){
      main = paste("mu=",mu," (",mainvec[case],")",sep="")
      colvar <- which(colnames(data)==var[case])
      for (p in 1:ncol){
        rows <- which((data[,"method"]==method[1]) & (data[,"mu"]==mu[p]))
        x <- data[rows,"T"]
        y <- data[rows,colvar]
        if (length(method)>1){
          for (i in 2:length(method)){
            rows <- which((data[,"method"]==method[i]) & (data[,"mu"]==mu[p]))
            y <- cbind(y,data[rows,colvar])
          }
        }
        pos <- c(1,ceiling(length(x)/2),length(x))
        matplot(1:length(x),y,type="l",lwd=2,lty=c(2,3,1),col=c(1,1,1),xlab="observations",ylab="abs. U",xaxt='n',main=main[p],mu[p],cex.main=1.5,cex.axis=1.2,cex.lab=1.5,ylim=c(-0.03,0.10))
        abline(a=0,b=0)
        axis(1,at=pos,labels=x[pos],cex.axis=1.2)
      }
    }
    par(mai=rep(0,4))
    plot.new()
    legend(x="center", ncol=3,legend=legendtext,lwd=2,lty=c(2,3,1),col=c(1,1,1),cex=1.5)
    if (plotsave & !onefile) dev.off()
  }
  
  # -------------------------------------------------------------------------
  # Differences utility (relative)
  for (k in 1:2){
    if (k==1){
      if (plotsave & !onefile) pdf(paste("Ur_",savenme.part,"_diff_probit.pdf",sep=""),width=12,height=9)
      method_bench <- method.bm[k]
      method <- method.p[2:3]
      legendtext <- legend.p[2:3]
    }else{
      if (plotsave & !onefile) pdf(paste("Ur_",savenme.part,"_diff_logit.pdf",sep=""),width=12,height=9)
      method_bench <- method.bm[k]
      method <- method.l[2:3]
      legendtext <- legend.l[2:3]
    }
    
    var <- c("Ur_in_m","Ur_out_m")
    mainvec <- c("is", "oos")
    
    layout(matrix(c(1:(2*ncol),rep(2*ncol+1,ncol)), ncol=ncol, byrow=TRUE), heights=c(4,4,1))
    par(mai=c(0.5,0.6,1,0.25))
    for (case in 1:2){
      main = paste("mu=",mu," (",mainvec[case],")",sep="")
      colvar <- which(colnames(data)==var[case])
      for (p in 1:ncol){
        rows <- which((data[,"method"]==method[1]) & (data[,"mu"]==mu[p]))
        rows_bench <- which((data[,"method"]==method_bench) & (data[,"mu"]==mu[p]))
        x <- data[rows,"T"]
        y <- data[rows,colvar] - data[rows_bench,colvar]
        if (length(method)>1){
          for (i in 2:length(method)){
            rows <- which((data[,"method"]==method[i]) & (data[,"mu"]==mu[p]))
            y <- cbind(y,data[rows,colvar] - data[rows_bench,colvar])
          }
        }
        pos <- c(1,ceiling(length(x)/2),length(x))
        matplot(1:length(x),y,type="l",lwd=2,lty=c(3,1),col=c(1,1,1),xlab="observations",ylab="Difference of rel. U",xaxt='n',main=main[p],mu[p],cex.main=1.7,cex.axis=1.5,cex.lab=1.7,ylim=c(-0.2,0.6))
        abline(a=0,b=0)
        axis(1,at=pos,labels=x[pos],cex.axis=1.2)
      }
    }
    par(mai=rep(0,4))
    plot.new()
    legend(x="center", ncol=2,legend=legendtext,lwd=2,lty=c(3,1),col=c(1,1,1),cex=1.7)
    if (plotsave & !onefile) dev.off()
  }
  
  # -------------------------------------------------------------------------
  # Differences utility (absolute)
  for (k in 1:2){
    if (k==1){
      if (plotsave & !onefile) pdf(paste("Ua_",savenme.part,"_diff_probit.pdf",sep=""),width=13,height=9)
      method_bench <- method.bm[k]
      method <- method.p[2:3]
      legendtext <- legend.p[2:3]
    }else{
      if (plotsave & !onefile) pdf(paste("Ua_",savenme.part,"_diff_logit.pdf",sep=""),width=13,height=9)
      method_bench <- method.bm[k]
      method <- method.l[2:3]
      legendtext <- legend.l[2:3]
    }
    
    var <- c("Ua_in_m","Ua_out_m")
    mainvec <- c("is", "oos")
    
    layout(matrix(c(1:(2*ncol),rep(2*ncol+1,ncol)), ncol=ncol, byrow=TRUE), heights=c(4,4,1))
    par(mai=c(0.5,0.6,1,0.25))
    for (case in 1:2){
      main = paste("mu=",mu," (",mainvec[case],")",sep="")
      colvar <- which(colnames(data)==var[case])
      for (p in 1:ncol){
        rows <- which((data[,"method"]==method[1]) & (data[,"mu"]==mu[p]))
        rows_bench <- which((data[,"method"]==method_bench) & (data[,"mu"]==mu[p]))
        x <- data[rows,"T"]
        y <- data[rows,colvar] - data[rows_bench,colvar]
        if (length(method)>1){
          for (i in 2:length(method)){
            rows <- which((data[,"method"]==method[i]) & (data[,"mu"]==mu[p]))
            y <- cbind(y,data[rows,colvar] - data[rows_bench,colvar])
          }
        }
        pos <- c(1,ceiling(length(x)/2),length(x))
        matplot(1:length(x),y,type="l",lwd=2,lty=c(3,1),col=c(1,1,1),xlab="observations",ylab="Difference of abs. U",xaxt='n',main=main[p],mu[p],cex.main=1.5,cex.axis=1.2,cex.lab=1.5,ylim=c(-0.02,0.02))
        abline(a=0,b=0)
        axis(1,at=pos,labels=x[pos],cex.axis=1.2)
      }
    }
    par(mai=rep(0,4))
    plot.new()
    legend(x="center", ncol=2,legend=legendtext,lwd=2,lty=c(3,1),col=c(1,1,1),cex=1.5)
    if (plotsave & !onefile) dev.off()
  }
  
  # -------------------------------------------------------------------------
  # Probability of outperformance utility (relative)
  for (k in 1:2){
    if (k==1){
      if (plotsave & !onefile) pdf(paste("probplot_",savenme.part,"_probit.pdf",sep=""),width=13,height=5)
      method <- method.p[2:3]
      legendtext <- legend.p[2:3]
    }else{
      if (plotsave & !onefile) pdf(paste("probplot_",savenme.part,"_logit.pdf",sep=""),width=13,height=5)
      method <- method.l[2:3]
      legendtext <- legend.l[2:3]
    }
    
    var <- "p_Ur_out"
    colvar <- which(colnames(data)==var)

    layout(matrix(c(1:ncol,rep(ncol+1,ncol)), ncol=ncol, byrow=TRUE), heights=c(4,1))
    par(mai=c(0.5,0.6,1,0.25))
    main = paste("mu=",mu,sep="")
    for (p in 1:ncol){
      rows <- which((data[,"method"]==method[1]) & (data[,"mu"]==mu[p]))
      x <- data[rows,"T"]
      y <- data[rows,colvar]
      if (length(method)>1){
        for (i in 2:length(method)){
          rows <- which((data[,"method"]==method[i]) & (data[,"mu"]==mu[p]))
          y <- cbind(y,data[rows,colvar])
        }
      }
      pos <- c(1,ceiling(length(x)/2),length(x))
      matplot(1:length(x),y,type="l",lwd=2,lty=c(3,1),col=c(1,1,1),xlab="observations",ylab="Prob of higher U",xaxt='n',main=main[p],mu[p],cex.main=1.5,cex.axis=1.2,cex.lab=1.5,ylim=c(0,1))
      abline(a=0.5,b=0)
      axis(1,at=pos,labels=x[pos],cex.axis=1.2)
    }
    
    par(mai=rep(0,4))
    plot.new()
    legend(x="center", ncol=2,legend=legendtext,lwd=2,lty=c(3,1),col=c(1,1,1),cex=1.5)
    if (plotsave & !onefile) dev.off()
  }
  
  if (plotsave & onefile) dev.off()
}