
randomdata <- function(T=1000,b,corr=FALSE,autocorr=FALSE,dist="norm"){
  #DESCRIPTION
  # Simulates observations from a data generating process
  # DGP is always given by y = P(x*b>0), with x random
  #-------------------------------------------------------------------------------
  #ARGUMENTS
  # T:            Number of observations
  # b:            linear multiplier on RHS-variables x
  # corr:         if TRUE, simulate x correlated with correlation of 0.5
  # autocorr:     if TRUE, simulate x as autocorrelated process with lag coefficients rho_1 = 0.7 and rho_2 = -0.1
  #-------------------------------------------------------------------------------
  # Gregor von Schweinitz and Peter Sarlin, 2018-07-25
  
  library("MASS")
  add <- 100
  Tlong <- T+add
  K<-length(b)-1
  
  mu <- rep(0,K)
  Sigma <- diag(K)
  cholS <- Sigma
  if (corr){
    l <- rep(0.5,K*(K-1)/2)
    Sigma[lower.tri(Sigma)] <- l
    Sigma[upper.tri(Sigma)] <- t(Sigma[lower.tri(Sigma)])
    cholS <- chol(Sigma)
    Sigma <- diag(K)
  }
  
  if (!autocorr){
    if (dist=="norm"){X <- matrix(rnorm(K*T),T,K) %*% cholS + mu}
    if (dist=="cauchy"){X <- matrix(rcauchy(K*T),T,K) %*% cholS + mu}
    if (dist=="exp"){X <- matrix(rexp(K*T),T,K) %*% cholS + mu}

    X <- data.frame(cbind(X,rep(1,T)))}
  else{ 
    # X autocorrelation
    rho <- c(0.7,-0.1)
    lags <- length(rho)
    X <- mat.or.vec(Tlong,K)
    X[1:lags,] <- matrix(rnorm(K*lags),lags,K) %*% cholS + mu
    for (i in (lags+1):Tlong){
      X[i,] <- rho%*%X[(i-lags):(i-1),] + matrix(rnorm(K*1),1,K) %*% cholS + mu
    }
    X <- cbind(X,rep(1,Tlong))
    X <- X[(add+1):Tlong,]
  }
  
  colnames(X) <- c(1:K,"Intercept")
  
  pb <- pnorm(as.matrix(X)%*%b)
  data <- data.frame(Date=1:T,Country=1,B=(runif(T)<pb)*1,X)
  LHS <- c("B")
  model <- colnames(data[,4:(K+4)])
  data_obj <- NULL
  data_obj$data <- data
  data_obj$LHS <- c("B")
  data_obj$model <- model
  return(data_obj)
}