#------------------------------------------------
# running regressions with real data
# Gregor von Schweinitz and Peter Sarlin, 2018-07-25

wd <- getwd()
source(paste(wd,"/construct_restable.fc.R",sep=""))
source(paste(wd,"/findThreshold.R",sep=""))
source(paste(wd,"/gr_wlogit.R",sep=""))
source(paste(wd,"/gr_wprobit.R",sep=""))
source(paste(wd,"/hess_wlogit.R",sep=""))
source(paste(wd,"/hess_wprobit.R",sep=""))
source(paste(wd,"/LL_wlogit.R",sep=""))
source(paste(wd,"/LL_wprobit.R",sep=""))
# source(paste(wd,"/mult_testing.R",sep=""))
# source(paste(wd,"/mult_testing_par.R",sep=""))
# source(paste(wd,"/mult_testing_signals_par.R",sep=""))
source(paste(wd,"/plot_U_realdata.R",sep=""))
source(paste(wd,"/plot_realtime.R",sep=""))
source(paste(wd,"/plot_U_bs_realdata.R",sep=""))
source(paste(wd,"/plot_crisisfreq.R",sep=""))
source(paste(wd,"/randomdata.R",sep=""))
# source(paste(wd,"/Ur_varmu.R",sep=""))
source(paste(wd,"/weighttest_realtime.R",sep=""))
# source(paste(wd,"/weighttesting_cv.R",sep=""))
source(paste(wd,"/weighttesting_bs.R",sep=""))
source(paste(wd,"/weighttesting_bs_run.R",sep=""))
source(paste(wd,"/weighttesting_thresh.R",sep=""))
#------------------------------------------------
# running regressions with BP or LDP data
# mu going in many steps

library("plyr")

# Preparing dataset

runs_bs <- 100
case <- "BP"
if (case=="LDP"){
  data_full <- read.csv("LDP_data_train_test_new.csv")
  data_full$dateid <- as.Date(paste(data_full[,"year"],3*data_full[,"quarter"]-2,"01",sep="-"))
  freq_cr <- ddply(data_full,.(dateid),summarize,freq_cr=mean(C18))
  plot_crisisfreq(freq_cr,savename="freq_crisis_BP",samplesplit="1995-05-01",ylab="freq (pre-crisis)",samplemeans=c(mean(data_full[data_full$MF=="TRAIN","C24"]),mean(data_full[data_full$MF!="TRAIN","C24"])))
  pos_use <- (data_full$C0==0) & (data_full$Exclude_C18==0)
  data <- data_full[pos_use,]
  model <- c("Inflation", "Real.GDP.growth", "Real.credit.growth", "Real.equity.growth", "Leverage", "Equity.valuation", "CA.deficit", "Government.deficit", "Global.inflation", "Global.real.GDP.growth", "Global.real.credit.growth", "Global.real.equity.growth", "Global.leverage", "Global.equity.valuation", "Intercept")
  
  #Definition of interactions for "benchmark model"
  data$Equity.Interaction <- data$Real.equity.growth*data$Equity.valuation
  data$Credit.Interaction <- data$Real.credit.growth*data$Leverage
  data$Global.Equity.Interaction <- data$Global.real.equity.growth*data$Global.equity.valuation
  data$Global.Credit.Interaction <- data$Global.real.credit.growth*data$Global.leverage
  data$Leverage.Interaction <- data$Global.leverage*data$Leverage
  data$Equity.valuation.Interaction <- data$Global.equity.valuation*data$Equity.valuation
  data$Real.credit.growth.Interaction <- data$Global.real.credit.growth*data$Real.credit.growth
  data$Real.equity.growth.Interaction <- data$Global.real.equity.growth*data$Real.equity.growth
  data$Equity.Double.Interaction<-data$Equity.Interaction*data$Global.Equity.Interaction
  data$Credit.Double.Interaction<-data$Credit.Interaction*data$Global.Credit.Interaction
  data$Intercept <- 1
  model <- colnames(data)[c(15,14,20,21,17,19,33,16,18,34,23,22,25,27,35,24,26,36,37:43)]
  
  countryname <- c("country")
  datename <- c("dateid")
  horizon <- 6
  LHS <- "C18"
  filename <- "realdata_LDP.RData"
  report <- "logit"
  reportcols <- c(5,7,6)
  method <- c("Logit opt threshold","Weighted logit","Logit set threshold")
  legendtext <- c("Logit, opt threshold","Weighted logit","Logit, set threshold")
  
  name_restable_oos <- "perf_oos_LDP.csv"
  name_restable_oos_AD <- "perf_oos_LDP_AD.csv"
}
if (case=="BP"){
  data_full <- read.csv("BP_data.csv")
  data <- data_full
  data$dateid <- as.Date(paste(unlist(lapply(strsplit(as.character(data[,"DATE"]),"M"),paste,collapse="-")),"-01",sep=""))
  freq_cr <- ddply(data,.(dateid),summarize,freq_cr=mean(C24))
  plot_crisisfreq(freq_cr,savename="freq_crisis_BP",samplesplit="1995-05-01",ylab="freq (pre-crisis)",samplemeans=c(mean(data_full[data_full$MF=="TRAIN","C24"]),mean(data_full[data_full$MF!="TRAIN","C24"])))
  data$Intercept <- 1
  countryname <- c("COUNTRY")
  datename <- c("dateid")
  model <- c("RESG","EXPG","RDEV","CANE","STDR2","Intercept")
  horizon <- 24
  LHS <- "C24"
  filename <- "realdata_BP.RData"
  report <- "probit"
  reportcols <- c(2,4,3)
  method <- c("Probit opt threshold","Weighted probit","Probit set threshold")
  legendtext <- c("Probit, opt threshold","Weighted probit","Probit, set threshold")
  
  name_restable_oos <- "perf_oos_BP.csv"
  name_restable_oos_AD <- "perf_oos_BP_AD.csv"
}

for (i in length(model)){data[,model[i]] <- as.numeric(data[,model[i]])}
rm(i)
data <- data[rowSums(is.na(data[,c(LHS,model)]))==0,]
data_obj <- list(data=data,LHS=LHS,model=model,subset=which(data[,"MF"]=="TRAIN"),countryname=countryname,datename=datename)
muvec <- seq(0.01,0.99,0.01)
muvec.fc <- c(0.2,0.5,0.8,0.95)
thetavec <- seq(0.01,0.99,0.01)
thetavec.fc <- c(0.3,0.5,0.7)
save.image(filename)

# running blockwise regressions
res_compare <- weighttesting_thresh(data_obj,mu=muvec,long=FALSE,oos=TRUE)
tab_Ur_is <-  cbind(mu=muvec,res_compare[,,"Ur_in"])
tab_Ur_oos <-  cbind(mu=muvec,res_compare[,,"Ur_out"])
tab_Ua_is <-  cbind(mu=muvec,res_compare[,,"Ua_in"])
tab_Ua_oos <-  cbind(mu=muvec,res_compare[,,"Ua_out"])
rm(res_compare)
save.image(filename)

#same again for AD preferences
res_compare <- weighttesting_thresh(data_obj,mu=muvec,long=FALSE,oos=TRUE,b_U_Sarlin=FALSE)
tab_Ur_is_AD <-  cbind(mu=muvec,res_compare[,,"Ur_in"])
tab_Ur_oos_AD <-  cbind(mu=muvec,res_compare[,,"Ur_out"])
tab_Ua_is_AD <-  cbind(mu=muvec,res_compare[,,"Ua_in"])
tab_Ua_oos_AD <-  cbind(mu=muvec,res_compare[,,"Ua_out"])
rm(res_compare)
save.image(filename)

# running blockwise regressions (restricted sample, LDP only)
if (case=="LDP"){
  pos<- data$dateid<="2007-04-01"
  data_rest <- data[pos,]
  data_obj_rest <- list(data=data_rest,LHS=LHS,model=model,subset=which(data_rest[,"MF"]=="TRAIN"),countryname=countryname,datename=datename)
  
  # Sarlin preferences
  res_compare <- weighttesting_thresh(data_obj_rest,mu=muvec,long=FALSE,oos=TRUE)
  tabrest_Ur_is <-  cbind(mu=muvec,res_compare[,,"Ur_in"])
  tabrest_Ur_oos <-  cbind(mu=muvec,res_compare[,,"Ur_out"])
  tabrest_Ua_is <-  cbind(mu=muvec,res_compare[,,"Ua_in"])
  tabrest_Ua_oos <-  cbind(mu=muvec,res_compare[,,"Ua_out"])
  rm(res_compare)
  save.image(filename)
  
  #same again for AD preferences
  res_compare <- weighttesting_thresh(data_obj_rest,mu=muvec,long=FALSE,oos=TRUE,b_U_Sarlin=FALSE)
  tabrest_Ur_is_AD <-  cbind(mu=muvec,res_compare[,,"Ur_in"])
  tabrest_Ur_oos_AD <-  cbind(mu=muvec,res_compare[,,"Ur_out"])
  tabrest_Ua_is_AD <-  cbind(mu=muvec,res_compare[,,"Ua_in"])
  tabrest_Ua_oos_AD <-  cbind(mu=muvec,res_compare[,,"Ua_out"])
  rm(res_compare)
  save.image(filename)
  
}

mutemp <- seq(0.2,0.8,0.1)
diff_Ur_oos <- mat.or.vec(length(mutemp),3)
if (case=="LDP"){
  tab <- tabrest_Ur_oos
  tab_is <- tabrest_Ur_is
}
if (case=="BP") {
  tab <- tab_Ur_oos
  tab_is <- tab_Ur_is
}
for (k in 1:length(mutemp)){
  pos <- which(tab[,1]<=mutemp[k])
  pos1 <- which(tab_is[pos,3]>=0)
  pos2 <- which(tab_is[pos,4]>=0)
  diff <- c(mean(tab[pos1,reportcols[2]]-tab[pos1,reportcols[1]]),mean(tab[pos2,reportcols[3]]-tab[pos2,reportcols[1]]))
  diff_Ur_oos[k,] <- c(mutemp[k],diff)
}
colnames(diff_Ur_oos) <- c("maximum mu",colnames(tab)[reportcols[2:3]])
rm(k,pos,pos1,pos2,diff,mutemp,tab,tab_is)
save.image(filename)


#------------------------------------------------
# Plotting blockwise Utilities
if (case=="LDP"){
  plot_U_realdata(filename="Real_LDP_Ur_rest", main = c("rel usefulness, 1990Q1-2005Q1, LDP","rel usefulness, 2005Q2-2007Q2, LDP"),
              tis = tabrest_Ur_is, toos = tabrest_Ur_oos,type="tiff")

  plot_U_realdata(filename="Real_LDP_Ua_rest", main = c("abs usefulness, 1990Q1-2005Q1, LDP","abs usefulness, 2005Q2-2007Q2, LDP"),
              tis = tabrest_Ua_is, toos = tabrest_Ua_oos,type="tiff")

  plot_U_realdata(filename="Real_LDP_Ur", main = c("rel usefulness, 1990Q1-2005Q1, LDP","rel usefulness, 2005Q2-2009Q2, LDP"),
              tis = tab_Ur_is, toos = tab_Ur_oos,type="tiff")

  plot_U_realdata(filename="Real_LDP_Ua", main = c("abs usefulness, 1990Q1-2005Q1, LDP","abs usefulness, 2005Q2-2009Q2, LDP"),
              tis = tab_Ua_is, toos = tab_Ua_oos,type="tiff")
  
  plot_U_realdata(filename="Real_LDP_Ur_AD_rest", main = c("rel usefulness AD, 1990Q1-2005Q1, LDP","rel usefulness AD, 2005Q2-2007Q2, LDP"),
                  tis = tabrest_Ur_is_AD, toos = tabrest_Ur_oos_AD,type="tiff")
  
  plot_U_realdata(filename="Real_LDP_Ua_AD_rest", main = c("abs usefulness AD, 1990Q1-2005Q1, LDP","abs usefulness AD, 2005Q2-2007Q2, LDP"),
                  tis = tabrest_Ua_is, toos = tabrest_Ua_oos,type="tiff")
  
  plot_U_realdata(filename="Real_LDP_Ur_AD", main = c("rel usefulness AD, 1990Q1-2005Q1, LDP","rel usefulness AD, 2005Q2-2009Q2, LDP"),
                  tis = tab_Ur_is_AD, toos = tab_Ur_oos_AD,type="tiff")
  
  plot_U_realdata(filename="Real_LDP_Ua_AD", main = c("abs usefulness AD, 1990Q1-2005Q1, LDP","abs usefulness AD, 2005Q2-2009Q2, LDP"),
                  tis = tab_Ua_is_AD, toos = tab_Ua_oos_AD,type="tiff")
}
if (case=="BP"){
  plot_U_realdata(filename="Real_BP_Ur", main = c("rel usefulness, is, BP","rel usefulness, oos, BP"),
                tis = tab_Ur_is, toos = tab_Ur_oos,type="tiff")
  
  plot_U_realdata(filename="Real_BP_Ua", main = c("abs usefulness, is, BP","abs usefulness, oos, BP"),
                tis = tab_Ua_is, toos = tab_Ua_oos,type="tiff")
  
  plot_U_realdata(filename="Real_BP_Ur_AD", main = c("rel usefulness AD, is, BP","rel usefulness AD, oos, BP"),
                  tis = tab_Ur_is_AD, toos = tab_Ur_oos_AD,type="tiff")
  
  plot_U_realdata(filename="Real_BP_Ua", main = c("abs usefulness AD, is, BP","abs usefulness AD, oos, BP"),
                  tis = tab_Ua_is_AD, toos = tab_Ua_oos_AD,type="tiff")
}

#############################################################################################################
# running real time regressions

print("real-time forecast U-Sarlin")
res_array_fc <- weighttest_realtime(data_obj,horizon,muvec=muvec,oos_oneperiod=TRUE)
res_array_fc_bs <- weighttesting_bs(data_obj=data_obj,runs = runs_bs,muvec=muvec.fc,alphavec=c(0.05,0.95),b_U_Sarlin=TRUE,realtime=TRUE,oos=TRUE)
save.image(filename)

print("real-time forecast U-AD")
res_array_fc_AD <- weighttest_realtime(data_obj,horizon,muvec=thetavec,oos_oneperiod=TRUE,b_U_Sarlin=FALSE)
res_array_fc_AD_bs <- weighttesting_bs(data_obj=data_obj,runs = runs_bs,muvec=thetavec.fc,alphavec=c(0.05,0.95),b_U_Sarlin=FALSE,realtime=TRUE,oos=TRUE)
save.image(filename)

restable_oos <- construct_restable.fc(res_array_fc,muvec.fc,res_array_fc_bs[,,"p"],b_U_Sarlin = TRUE,savenme=name_restable_oos)
restable_oos_AD <- construct_restable.fc(res_array_fc_AD,thetavec.fc,res_array_fc_AD_bs[,,"p"],b_U_Sarlin = FALSE,savenme=name_restable_oos_AD)
save.image(filename)

# Calculating statistics
resvec <- c("A_out","B_out","C_out","D_out","Ua_out","Ur_out")
res_array_av_oos <- array(dim=c(length(muvec),6,length(resvec)),
                          dimnames=list(mu=muvec,method=dimnames(res_array_fc)$method,res=resvec))
pos_time <- dimnames(res_array_fc)$T<="2007-10-01"

N <- sum(res_array_fc[pos_time,1,1,c("A_out","B_out","C_out","D_out")])
P1 <- sum(res_array_fc[pos_time,1,1,c("A_out","C_out")])/N
P2 <- sum(res_array_fc[pos_time,1,1,c("B_out","D_out")])/N
for (k in 1:length(muvec)){
  for (n in 1:length(dimnames(res_array_fc)$method)){
    for (res in 1:4){
      res_array_av_oos[k,n,resvec[res]] <- sum(res_array_fc[pos_time,n,k,resvec[res]])
    }
  }
  Ua_best <- min(muvec[k]*P1,(1-muvec[k])*P2)
  res_array_av_oos[k,,"Ua_out"] <- Ua_best - (muvec[k]*res_array_av_oos[k,,"C_out"]/N + (1-muvec[k])*res_array_av_oos[k,,"B_out"]/N)
  res_array_av_oos[k,,"Ur_out"] <- res_array_av_oos[k,,"Ua_out"]/Ua_best
}
rm(resvec,N,P1,P2,k,n,Ua_best)
save.image(filename)

#------------------------------------------------
# plotting lambda heatplot and real-time usefulness
if (case=="LDP"){
  plot_realtime(res_array=res_array_fc,
                filename1="lambda_RT_LDP_logit",column=4,muvec1=muvec,ylab="mu",
                filename2="Ua_RT_LDP_logit",
                method=method,
                legendtext=legendtext,
                var="Ua_out",mu=muvec.fc,
                filename3="freq_thresholds_LDP_logit",freq_cr=freq_cr[freq_cr[,1]%in%as.Date(dimnames(res_array_fc)[[1]]),2],b_U_Sarlin = TRUE,type="tiff")
  plot_realtime(res_array=res_array_fc_AD,
                filename1="lambda_RT_LDP_logit_AD",column=4,muvec1=muvec,ylab="theta",
                filename2="Ua_RT_LDP_logit_AD",
                method=method,
                legendtext=legendtext,
                var="Ua_out",mu=thetavec.fc,
                filename3="freq_thresholds_LDP_logit_AD",freq_cr=freq_cr[freq_cr[,1]%in%as.Date(dimnames(res_array_fc_AD)[[1]]),2],b_U_Sarlin = FALSE,type="tiff")
}
if (case=="BP"){
  plot_realtime(res_array=res_array_fc,
                filename1="lambda_RT_BP_probit",column=1,muvec1=muvec,ylab="mu",
                filename2="Ua_RT_BP_probit",
                method=method,
                legendtext=legendtext,
                var="Ua_out",mu=muvec.fc,
                filename3="freq_thresholds_BP_probit",freq_cr=freq_cr[freq_cr[,1]%in%as.Date(dimnames(res_array_fc)[[1]]),2],b_U_Sarlin = TRUE,type="tiff")
  plot_realtime(res_array=res_array_fc_AD,
                filename1="lambda_RT_BP_probit_AD",column=1,muvec1=muvec,ylab="theta",
                filename2="Ua_RT_BP_probit_AD",
                method=method,
                legendtext=legendtext,
                var="Ua_out",mu=thetavec.fc,
                filename3="freq_thresholds_BP_probit_AD",freq_cr=freq_cr[freq_cr[,1]%in%as.Date(dimnames(res_array_fc_AD)[[1]]),2],b_U_Sarlin = FALSE,type="tiff")
}


#############################################################################################################
# running bootstrap exercice
# goal: return mean usefulness, confidence bands, and p-values of out-performance for all methods

res_array_bs <- weighttesting_bs(data_obj=data_obj,runs = runs_bs,muvec=muvec,alphavec=c(0.05,0.95),b_U_Sarlin=TRUE)
save.image(filename)
res_array_bs_AD <- weighttesting_bs(data_obj=data_obj,runs = runs_bs,muvec=muvec,alphavec=c(0.05,0.95),b_U_Sarlin=FALSE)
save.image(filename)
plot_prob_bs_realdata(filename=paste("prob_oos_bs_",case,sep=""),main=paste("Probability of outperformance oos,",c("Sarlin","AD")),
                      res_array=res_array_bs,res_array_AD=res_array_bs_AD,ylab="prob(outperformance)",xlab="mu",type="tiff")
plot_U_bs_realdata(filename=paste("Ur_bs_",case,sep=""),main=c(paste("Ur oos,",case),"Probability of outperformance"),
                   res_mean = tab_Ur_oos,res_array=res_array_bs,ylab="Ur",xlab="mu",type="tiff")
plot_U_bs_realdata(filename=paste("Ur_AD_bs_",case,sep=""),main=c(paste("Ur (AD) oos,",case),"Probability of outperformance"),
                   res_mean = tab_Ur_oos_AD,res_array=res_array_bs_AD,ylab="Ur",xlab="theta",type="tiff")




