weighttesting_bs <- function(data_obj,runs,muvec,alphavec,realtime=FALSE,oos=TRUE,datetype="quarters",blocklength=12,sample.countries=FALSE,b_U_Sarlin=TRUE){
  # Sets up bootstrap exercise on real data. Wrapper function as setting up of the bootstrap can be done once in a recursive exercise
  #
  # Gregor von Schweinitz and Peter Sarlin, 2018-07-25
  
  library(foreach)
  library(doParallel)
  library(plyr)
  library(optimx)
  cl<-makeCluster(12)
  registerDoParallel(cl)
  set.seed(111)
  
  
  if (realtime & oos){
    datename <- data_obj$datename
    countryname <- data_obj$countryname
    data_full <- data_obj$data
    countries <- unique(data_obj$data[,countryname])
    dates <- sort(unique(data_obj$data[,datename]))
    Tvec <- 1:length(dates)
    Tvec_is <- 1:length(unique(data_full[data_obj$subset,datename]))
    Tvec_oos <- Tvec[-Tvec_is]
    
    tab_Y <- NA*mat.or.vec(length(dates),length(countries))
    rownames(tab_Y) <- Tvec
    colnames(tab_Y) <- countries
    tab_pos <- tab_Y
    tab_use <- tab_Y
    for (k in 1:length(countries)){
      countrypos <- which(data_full[,countryname]==countries[k])
      temp <- data_full[countrypos,c(datename,data_obj$LHS)]
      rows <- which(dates%in%temp[,datename])
      tab_Y[rows,k] <- temp[,data_obj$LHS]
      tab_pos[rows,k] <- countrypos
      tab_use[,k] <- Tvec+horizon
      pos_cr <- which(diff(tab_Y[,k])==-1)+1  #position of true crises (after EWW)
      for (p in pos_cr){tab_use[max(p-horizon,1):(p),k] <- p}
    }
    pos_NA <- is.na(tab_Y)
    tab_use[pos_NA] <- NA
    
    for (k in 1:length(Tvec_oos)){
      T <- Tvec_oos[k]
      print(paste("Out-of-sample for T =",dates[T]))
      pos_is <- tab_pos[tab_use<T & !is.na(tab_use)]
      pos_oos <- which(data_full[,datename]==dates[T])
      
      data_obj_bs <- data_obj
      data_obj_bs$subset <- 1:length(pos_is)
      data_obj_bs$data <- rbind(data_full[pos_is,],data_full[pos_oos,])
      long.temp <- weighttesting_bs_run(data_obj=data_obj_bs,muvec=muvec,
                                        runs=runs,datetype=datetype,blocklength=blocklength,sample.countries=sample.countries,
                                        b_U_Sarlin=b_U_Sarlin)
      
      if (k==1){
        long <- long.temp
      }else{
        long.temp <- long.temp[,,c("A_out","B_out","C_out","D_out"),]
        long.temp[is.na(long.temp)] <- 0
        long[,,c("A_out","B_out","C_out","D_out"),] <- long[,,c("A_out","B_out","C_out","D_out"),]+long.temp
        
        
        
        mu <- array(muvec,dim=dim(long)[-3])
        N <- sum(long[1,1,c("A_out","B_out","C_out","D_out"),1])
        P1N <- sum(long[1,1,c("A_out","C_out"),1])
        P2N <- sum(long[1,1,c("B_out","D_out"),1])
        if (b_U_Sarlin){
          Umax <- pmin(mu*P1N/N,(1-mu)*P2N/N)
          L <- mu*long[,,"C_out",]/N+(1-mu)*long[,,"B_out",]/N  # Loss function
        }else{
          Umax <- pmin(mu,1-mu)
          L <- mu*long[,,"C_out",]/P1N+(1-mu)*long[,,"B_out",]/P2N
        }
        Ua <- Umax-L # absolute utility
        Ur <- Ua/Umax # rel utility
        long[,,"Ua_out",] <- Ua
        long[,,"Ur_out",] <- Ur
      }
      # long.T[k,,,,] <- long.temp
      # aggregate long.T to long!
    }
    # long <- apply(long.T,1,sum,na.rm=TRUE)
    

  }else{
    if (oos){
      data_obj_bs <- data_obj
    }else{
      #is estimation
      data_obj_bs <- data_obj
      data_obj_bs$subset <- 1:dim(data_obj_bs$data)[1]
      data_obj_bs$data <- rbind(data_obj_bs$data,data_obj_bs$data)
    }
    long <- weighttesting_bs_run(data_obj=data_obj_bs,muvec=muvec,
                                 runs=runs,datetype=datetype,blocklength=blocklength,sample.countries=sample.countries,
                                 b_U_Sarlin=b_U_Sarlin)
  }
  
  # long should be a four-dimensional array with dimensions
  #   mu, method, output, runs
  stopCluster(cl)
  
  
  mvec <- c("Probit opt threshold","Probit set threshold","Weighted probit",
            "Logit opt threshold","Logit set threshold","Weighted logit")
  m_bm <- c(1,1,1,4,4,4)
  resvec <- c("Ua",paste("Ua_",alphavec,sep=""),"Ur",paste("Ur_",alphavec,sep=""),"p")
  resarray <- array(0,dim=c(length(muvec),length(mvec),length(resvec)),
                    dimnames=list(mu=muvec,method=mvec,res=resvec))
  for (method in mvec){
    resarray[,method,"Ua"] <- apply(long[,method,"Ua_out",],1,quantile,0.5) #colMeans?? test that!!
    resarray[,method,"Ur"] <- apply(long[,method,"Ur_out",],1,quantile,0.5)
    for (alpha in alphavec){
      resarray[,method,paste("Ua_",alpha,sep="")] <- apply(long[,method,"Ua_out",],1,quantile,alpha)
      resarray[,method,paste("Ur_",alpha,sep="")] <- apply(long[,method,"Ur_out",],1,quantile,alpha)
    }
    resarray[,method,"p"] <- rowMeans(long[,method,"Ur_out",]>long[,m_bm[which(mvec==method)],"Ur_out",],na.rm=TRUE)
  }
  return(resarray)
}