weighttesting_bs_run <- function(data_obj,muvec,runs,datetype="quarters",blocklength=12,sample.countries=FALSE,b_U_Sarlin=TRUE){
  # Runs bootstrap exercise on real data, as described in the paper
  #
  # Gregor von Schweinitz and Peter Sarlin, 2018-07-25
  
  
  functionlist <- c("weighttesting_thresh","LL_wlogit","LL_wprobit","gr_wlogit","gr_wprobit","findThreshold")
  
  datename <- data_obj$datename
  countryname <- data_obj$countryname
  data.db.train <- data_obj$data[data_obj$subset,]
  data.db.test <- data_obj$data[-data_obj$subset,]
  
  N <- nrow(data.db.train)
  indices <- seq(1, N)
  P1 <- mean(data.db.train[,LHS],na.rm=TRUE)
  P2 <- 1-P1
  
  add.date= function(date,n) {
    if (datetype=="months") seq(date, by = paste (n, datetype), length = 2)[2]
    if (datetype=="quarters")  seq(date, by = paste (3*n, "months"), length = 2)[2]
  }
  add.date.v= function(date,n) as.Date(sapply(date, add.date, n), origin="1970-01-01")
  
  b <- blocklength
  time.vec <- unique(data.db.train[,datename])
  if (datetype=="months") time.part <- sort(seq(time.vec[1],by=paste("-1",datetype),length=b))
  if (datetype=="quarters")  time.part<- sort(seq(time.vec[1],by=paste("-3","months"),length=b))
  time.vec <- c(time.part,time.vec[2:length(time.vec)])
  sample.master <- data.frame(pos1=NA,pos2=NA,time.coun=as.Date("1970-01-01"),Country=NA)
  for (coun in unique(data.db.train[,countryname])){
    pos <- which(data.db.train[,countryname]==coun)
    pos1 <- c(rep(pos[1],b-1),pos)
    pos2 <- c(pos,rep(pos[length(pos)],b-1))
    time.coun <- data.db.train[pos,datename]
    if (datetype=="months") time.part <- sort(seq(time.coun[1],by=paste("-1",datetype),length=b))
    if (datetype=="quarters")  time.part<- sort(seq(time.coun[1],by=paste("-3","months"),length=b))
    time.coun1 <- data.db.train[pos1,datename]
    time.coun2 <- data.db.train[pos2,datename]
    
    pos.na <- which(add.date.v(time.coun1,b-1)<time.coun2)
    for (k in pos.na){
      pos2[k] <- pos[max(which(time.coun<=add.date(time.coun1[k],b-1)))]
    }
    time.coun2 <- data.db.train[pos2,datename]
    time.coun <- c(time.part,time.coun[2:length(time.coun)])
    sample.master <- rbind(sample.master,data.frame(pos1=pos1,pos2=pos2,time.coun=time.coun,Country=coun))
  }
  sample.master <- sample.master[2:dim(sample.master)[1],]
  rm(time.part,coun,pos,pos1,pos2,time.coun,time.coun1,time.coun2,pos.na)
  
  
  res_byrun <- function(r){
    #     print(r)
    data_obj_bs <- data_obj
    
    # randomly drawn countries (with replacement)
    if (sample.countries) {
      countries.bs <- sample(unique(data.db.train[,countryname]),replace=TRUE)
    }else{
      countries.bs <- unique(data.db.train[,countryname])
    }
    pos.bs1 <- unlist(lapply(countries.bs,grep,sample.master$Country))
    sample.bs <- sample.master[pos.bs1,]
    # randomly drawn starting times
    time.bs <- sample(time.vec,replace=TRUE)
    pos.bs2 <- unlist(lapply(time.bs,grep,sample.bs$time.coun))
    # get blocks from data.train
    pos.bs <- unlist(apply(sample.bs[pos.bs2,],1,function(x) seq(as.numeric(x["pos1"]),as.numeric(x["pos2"]))))
    
    T <- dim(data.db.train)[1]
    data_obj_bs$subset <- c(1:T)
    data_obj_bs$data <- rbind(data.db.train[pos.bs[1:T],],data.db.test)
    rm(countries.bs,pos.bs1,sample.bs,time.bs,pos.bs2,pos.bs)
    res_compare <- weighttesting_thresh(data_obj_bs,mu=muvec,long=FALSE,oos=TRUE,b_U_Sarlin=b_U_Sarlin)
    outmat <- res_compare[,,c("A_out","B_out","C_out","D_out","Ua_out","Ur_out")]
    return(outmat)
  }
  
  long <- foreach(r=1:runs,.export=functionlist) %dopar% {res_byrun(r)}
  # long <- foreach(r=1:runs,.export=functionlist) %do% {res_byrun(r)}
  long <- array(unlist(long),dim=c(dim(long[[1]]),length(long)),dimnames=c(dimnames(long[[1]]),list(1:runs)))
  return(long)
}